package com.ai.android.alarms;

import java.util.Calendar;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;

public class AlarmTester extends BaseTester 
{
	private static String tag = "AlarmTester";
	AlarmTester(Context ctx, IReportBack target)
	{
		super(ctx, target);
	}

	/*
	 * An alarm can invoke a broadcast request
	 * at a specified time.
	 * The name of the broadcast receiver is explicitly
	 * specified in the intent.
	 */
    public void sendAlarmOnce()
    {
    	//Get the instance in time that is 
    	//30 secs from now.
    	Calendar cal = Utils.getTimeAfterInSecs(30);
    	
    	//If you want to point to 11:00 hours today.
    	//Calendar cal = Utils.getTodayAt(11);

    	//Print to the debug view that we are 
    	//scheduling at a specific time
    	String s = Utils.getDateTimeString(cal);
    	mReportTo.reportBack(tag, "Schdeduling alarm at: " + s);
    	
    	//Get an intent to invoke
    	//TestReceiver class
    	Intent intent = 
    		new Intent(mContext, TestReceiver.class);
    	intent.putExtra("message", "Single Shot Alarm");

    	PendingIntent pi = 
    		PendingIntent.getBroadcast(
    		  mContext, 	//context
              1, 		    //request id, used for disambiguating this intent
              intent, 	    //intent to be delivered 
              PendingIntent.FLAG_ONE_SHOT);  //pending intent flags
    	// Schedule the alarm!
    	AlarmManager am = 
    		(AlarmManager)
    		     mContext.getSystemService(Context.ALARM_SERVICE);
    	
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal.getTimeInMillis(), 
    			pi);
    }
   
	/*
	 * An alarm can invoke a broadcast request
	 * starting at a specified time and at 
	 * regular intervals.
	 * 
	 * Uses the same intent as above
	 * but a distinct request id to avoid conflicts
	 * with the single shot alarm above.
	 * 
	 * Uses getDistinctPendingIntent() utility.
	 */
    public void sendRepeatingAlarm()
    {
    	Calendar cal = Utils.getTimeAfterInSecs(30);
    	//Calendar testcal = Utils.getTodayAt(11);
    	String s = Utils.getDateTimeString(cal);
    	this.mReportTo.reportBack(tag,
    		"Schdeduling Repeating alarm in 5 sec interval starting at: " + s);
    	
    	//Get an intent to invoke
    	//TestReceiver class
    	Intent intent = 
    		new Intent(this.mContext, TestReceiver.class);
    	intent.putExtra("message", "Repeating Alarm");

    	PendingIntent pi = this.getDistinctPendingIntent(intent, 2);
    	// Schedule the alarm!
    	AlarmManager am = 
    		(AlarmManager)
    		    this.mContext.getSystemService(Context.ALARM_SERVICE);
    	
    	am.setRepeating(AlarmManager.RTC_WAKEUP, 
    			cal.getTimeInMillis(), 
    			5*1000, //5 secs 
    			pi);
    }
    
	/*
	 * An alarm can be stopped by canceling the intent.
	 * You will need to have a copy of the intent
	 * to cancel it.
	 * 
	 * The intent needs to have the same signature
	 * and request id.
	 */
    public void cancelRepeatingAlarm()
    {
    	//Get an intent to invoke
    	//TestReceiver class
    	Intent intent = 
    		new Intent(this.mContext, TestReceiver.class);
    	
    	//To cancel, extra is not necessary to be filled in
    	//intent.putExtra("message", "Repeating Alarm");

    	PendingIntent pi = this.getDistinctPendingIntent(intent, 2);
    	
    	// Schedule the alarm!
    	AlarmManager am = 
    		(AlarmManager)
    		   this.mContext.getSystemService(Context.ALARM_SERVICE);
    	am.cancel(pi);
    	this.mReportTo.reportBack(tag,"You shouldn't see alarms again");
    }
    
	/*
	 * Same intent cannot be scheduled multiple times.
	 * If you do, only the last one will take affect.
	 * 
	 * Notice you are using the same request id.
	 */
    public void scheduleSameIntentMultipleTimes()
    {
    	//Get the instance in time that is 
    	//30 secs from now.
    	Calendar cal = Utils.getTimeAfterInSecs(30);
    	Calendar cal2 = Utils.getTimeAfterInSecs(35);
    	Calendar cal3 = Utils.getTimeAfterInSecs(40);
    	Calendar cal4 = Utils.getTimeAfterInSecs(45);
    	
    	//If you want to point to 11:00 hours today.
    	//Calendar cal = Utils.getTodayAt(11);

    	//Print to the debug view that we are 
    	//scheduling at a specific time
    	String s = Utils.getDateTimeString(cal);
    	mReportTo.reportBack(tag, "Schdeduling alarm at: " + s);
    	
    	//Get an intent to invoke
    	//TestReceiver class
    	Intent intent = 
    		new Intent(mContext, TestReceiver.class);
    	intent.putExtra("message", "Single Shot Alarm");

    	PendingIntent pi = this.getDistinctPendingIntent(intent, 1);
    	
    	// Schedule the alarm!
    	AlarmManager am = 
    		(AlarmManager)
    		     mContext.getSystemService(Context.ALARM_SERVICE);
    	
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal.getTimeInMillis(), 
    			pi);
    	
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal2.getTimeInMillis(), 
    			pi);
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal3.getTimeInMillis(), 
    			pi);
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal4.getTimeInMillis(), 
    			pi);
    }
	/*
	 * Same intent can be scheduled multiple times
	 * if you change the request id on the pending intent.
	 * Request id identifies an intent as a unique intent.
	 */
    public void scheduleDistinctIntents()
    {
    	//Get the instance in time that is 
    	//30 secs from now.
    	Calendar cal = Utils.getTimeAfterInSecs(30);
    	Calendar cal2 = Utils.getTimeAfterInSecs(35);
    	Calendar cal3 = Utils.getTimeAfterInSecs(40);
    	Calendar cal4 = Utils.getTimeAfterInSecs(45);
    	
    	//If you want to point to 11:00 hours today.
    	//Calendar cal = Utils.getTodayAt(11);

    	//Print to the debug view that we are 
    	//scheduling at a specific time
    	String s = Utils.getDateTimeString(cal);
    	mReportTo.reportBack(tag, "Schdeduling alarm at: " + s);
    	
    	//Get an intent to invoke
    	//TestReceiver class
    	Intent intent = 
    		new Intent(mContext, TestReceiver.class);
    	intent.putExtra("message", "Single Shot Alarm");

    	// Schedule the alarms!
    	AlarmManager am = 
    		(AlarmManager)
    		     mContext.getSystemService(Context.ALARM_SERVICE);
    	
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal.getTimeInMillis(), 
    			getDistinctPendingIntent(intent,1));
    	
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal2.getTimeInMillis(), 
    			getDistinctPendingIntent(intent,2));
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal3.getTimeInMillis(), 
    			getDistinctPendingIntent(intent,3));
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal4.getTimeInMillis(), 
    			getDistinctPendingIntent(intent,4));
    }
    
	/*
	 * It is not the alarm that matters 
	 * but the pending intent.
	 * Even with a repeating alarm for an intent,
	 * if you schedule the same intent again
	 * for one time, the later one takes affect.
	 * 
	 * It is as if you are setting the 
	 * alarm on an existing intent multiple
	 * times and not the other way around.
	 */
    public void alarmIntentPrimacy()
    {
    	Calendar cal = Utils.getTimeAfterInSecs(30);
    	String s = Utils.getDateTimeString(cal);
    	this.mReportTo.reportBack(tag,
    		"Schdeduling Repeating alarm in 5 sec interval starting at: " + s);
    	
    	//Get an intent to invoke
    	//TestReceiver class
    	Intent intent = 
    		new Intent(this.mContext, TestReceiver.class);
    	intent.putExtra("message", "Repeating Alarm");

    	PendingIntent pi = getDistinctPendingIntent(intent,0); 
    	AlarmManager am = 
    		(AlarmManager)
    		    this.mContext.getSystemService(Context.ALARM_SERVICE);
    	
    	this.mReportTo.reportBack(tag,"Setting a repeat alarm 5 secs duration");
    	am.setRepeating(AlarmManager.RTC_WAKEUP, 
    			cal.getTimeInMillis(), 
    			5*1000, //5 secs 
    			pi);

    	this.mReportTo.reportBack(tag,"Setting a onetime alarm on the same intent");
    	am.set(AlarmManager.RTC_WAKEUP, 
    			cal.getTimeInMillis(), 
    			pi);
    	this.mReportTo.reportBack(tag,"The later alarm, one time one, takes precedence");
    }
    private PendingIntent getDistinctPendingIntent(Intent intent, int requestId)
    {
    	PendingIntent pi = 
    		PendingIntent.getBroadcast(
    		  mContext, 	//context
              requestId, 	//request id
              intent, 		//intent to be delivered
              0);

        //pending intent flags
        //PendingIntent.FLAG_ONE_SHOT);    	
    	return pi;
    }
}
